Class: JsonapiCompliable::Configuration

Inherits:
Object
  • Object
show all
Defined in:
lib/jsonapi_compliable/configuration.rb

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize ⇒ Configuration

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Set defaults



10
11
12
# File 'lib/jsonapi_compliable/configuration.rb', line 10

def initialize
  @raise_on_missing_sideload = true
end

Instance Attribute Details

#raise_on_missing_sideload ⇒ Boolean

Returns Should we raise when the client requests a relationship not defined on the server? Defaults to true.

Returns:

  • (Boolean) —

    Should we raise when the client requests a relationship not defined on the server? Defaults to true.



6
7
8
# File 'lib/jsonapi_compliable/configuration.rb', line 6

def raise_on_missing_sideload
  @raise_on_missing_sideload
end